import { system } from "@minecraft/server";
import { Server, Thread } from "./../../library/Minecraft.js";
import { print, printerr } from "../util.js";
import { RawText } from "./../../library/Minecraft.js";
/**
 * The base tool class for handling tools that WorldEdit builders may use.
 */
export class Tool {
    constructor() {
        /**
         * Whether there should be some delay between item use to avoid rapid fire.
         */
        this.noDelay = false;
        this.useOnTick = 0;
        this.lastUse = system.currentTick;
    }
    log(message) {
        print(message, this.currentPlayer, true);
    }
    process(session, tick, loc, brokenBlock) {
        const player = session.getPlayer();
        if (!loc && !this.use || loc && !brokenBlock && !this.useOn || brokenBlock && !this.breakOn) {
            return false;
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const onFail = (e) => {
            printerr(e.message ? RawText.text(`${e.name}: `).append("translate", e.message) : e, player, true);
            if (e.stack) {
                printerr(e.stack, player, false);
            }
        };
        new Thread().start(function* (self, player, session, loc, brokenBlock) {
            self.currentPlayer = player;
            session.usingItem = true;
            try {
                if (!Server.player.hasPermission(player, self.permission)) {
                    throw "worldedit.tool.noPerm";
                }
                if (system.currentTick - self.lastUse > 4 || self.noDelay) {
                    self.lastUse = system.currentTick;
                    if (!loc) {
                        if (self.useOnTick != tick) {
                            if (self.use.constructor.name == "GeneratorFunction") {
                                yield* self.use(self, player, session);
                            }
                            else {
                                self.use(self, player, session);
                            }
                        }
                    }
                    else if (!brokenBlock) {
                        self.useOnTick = tick;
                        if (self.useOn.constructor.name == "GeneratorFunction") {
                            yield* self.useOn(self, player, session, loc);
                        }
                        else {
                            self.useOn(self, player, session, loc);
                        }
                    }
                    else {
                        self.breakOn(self, player, session, loc, brokenBlock);
                    }
                }
            }
            catch (e) {
                onFail(e);
            }
            finally {
                session.usingItem = false;
            }
            self.currentPlayer = null;
        }, this, player, session, loc, brokenBlock);
        return true;
    }
    delete() {
        return;
    }
    toJSON() {
        return { type: this.type };
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unused-vars
    static parseJSON(json) {
        return [];
    }
}
